(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-8b77bb46"],{"006b":function(t,e,r){"use strict";r.r(e);var o=function(){var t=this,e=t.$createElement,r=t._self._c||e;return r(t.rootComponent,{ref:"root",tag:"component",attrs:{template:t.contentTemplate}})},n=[],i=r("2909"),c=r("5530"),a=(r("2ca0"),r("d3b7"),r("3ca3"),r("ddb0"),r("2b3d"),r("9861"),r("159b"),function(t,e,r){if(!e.hasOwnProperty(r)){var o=Object.getOwnPropertyDescriptor(t,r);Object.defineProperty(e,r,o)}}),s={props:{template:String,parent:Object,templateProps:{type:Object,default:function(){return{}}}},render:function(t){if(this.template){var e=this.parent||this.$parent,r=e.$data;void 0===r&&(r={});var o=e.$props;void 0===o&&(o={});var n=e.$options;void 0===n&&(n={});var i=n.components;void 0===i&&(i={});var c=n.computed;void 0===c&&(c={});var s=n.methods;void 0===s&&(s={});var p=this.$data;void 0===p&&(p={});var u=this.$props;void 0===u&&(u={});var d=this.$options;void 0===d&&(d={});var f=d.methods;void 0===f&&(f={});var v=d.computed;void 0===v&&(v={});var h=d.components;void 0===h&&(h={});var l={$data:{},$props:{},$options:{},components:{},computed:{},methods:{}};Object.keys(r).forEach((function(t){void 0===p[t]&&(l.$data[t]=r[t])})),Object.keys(o).forEach((function(t){void 0===u[t]&&(l.$props[t]=o[t])})),Object.keys(s).forEach((function(t){void 0===f[t]&&(l.methods[t]=s[t])})),Object.keys(c).forEach((function(t){void 0===v[t]&&(l.computed[t]=c[t])})),Object.keys(i).forEach((function(t){void 0===h[t]&&(l.components[t]=i[t])}));var m=Object.keys(l.methods||{}),b=Object.keys(l.$data||{}),g=Object.keys(l.$props||{}),O=Object.keys(this.templateProps),P=b.concat(g).concat(m).concat(O),w=(j=e,E={},m.forEach((function(t){return a(j,E,t)})),E),y=function(t){var e={};return t.forEach((function(t){t&&Object.getOwnPropertyNames(t).forEach((function(r){return a(t,e,r)}))})),e}([l.$data,l.$props,w,this.templateProps]);return t({template:this.template||"<div></div>",props:P,computed:l.computed,components:l.components,provide:this.$parent._provided},{props:y})}var j,E}},p=r("2f62"),u=r("3800"),d={name:"App",components:{VRuntimeTemplate:s},computed:Object(c["a"])(Object(c["a"])(Object(c["a"])({},Object(u["b"])()),Object(p["c"])({get:"APP_GET",contentTemplate:"APP_TEMPLATE",wsClient:"WS_CLIENT",actions:"APP_ACTIONS",routes:"APP_ROUTES",tts:"APP_TEMPLATE_TS"})),{},{rootComponent:function(){return this.contentTemplate.startsWith("#")?this.contentTemplate.substring(1):"v-runtime-template"}}),watch:{actions:function(t){for(var e=0;e<t.length;e++){var r=t[e],o=r.ref,n=r.type;if("method"===n){var c,a=r.method,s=r.args;(c=this.getRef(o))[a].apply(c,Object(i["a"])(s))}if("property"===n){var p=r.property,u=r.value;this.getRef(o)[p]=u}}this.clearActions()}},data:function(){return{window:window}},methods:Object(c["a"])(Object(c["a"])({},Object(p["b"])({serverSet:"APP_SET",setAll:"APP_SET_ALL",serverTrigger:"APP_TRIGGER",clearActions:"APP_ACTIONS_PROCESSED",registerDecorator:"APP_REGISTER_DECORATOR",dirty:"APP_DIRTY"})),{},{set:function(t,e){this.serverSet({key:t,value:e})},trigger:function(t,e,r){this.serverTrigger({name:t,args:e,kwargs:r})},getRef:function(t){var e=this.$refs.root;return e.$refs&&e.$refs[t]?e.$refs[t]:e.$children[0].$refs[t]},download:function(t,e){var r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"application/octet-stream",o=new Blob([e],{type:r}),n=URL.createObjectURL(o),i=document.createElement("a");i.setAttribute("href",n),i.setAttribute("download",t),document.body.appendChild(i),i.click(),document.body.removeChild(i),setTimeout((function(){return URL.revokeObjectURL(n)}),1e3)}}),provide:function(){var t=this;return{get:function(e){return t.get(e)},set:function(e,r){return t.set(e,r)},setAll:function(e){return t.setAll(e)},trigger:function(e,r,o){return t.trigger(e,r,o)},dirty:function(e){return t.dirty(e)},wsClient:this.wsClient,isBusy:function(){return t.busy},window:this.window,registerDecorator:function(e){return t.registerDecorator(e)},download:function(e,r,o){return t.download(e,r,o)}}},mounted:function(){var t=this;this.routes.forEach((function(e){t.$router.addRoute(e)}))}},f=d,v=r("2877"),h=Object(v["a"])(f,o,n,!1,null,null,null);e["default"]=h.exports},"2ca0":function(t,e,r){"use strict";var o=r("23e7"),n=r("06cf").f,i=r("50c4"),c=r("577e"),a=r("5a34"),s=r("1d80"),p=r("ab13"),u=r("c430"),d="".startsWith,f=Math.min,v=p("startsWith"),h=!u&&!v&&!!function(){var t=n(String.prototype,"startsWith");return t&&!t.writable}();o({target:"String",proto:!0,forced:!h&&!v},{startsWith:function(t){var e=c(s(this));a(t);var r=i(f(arguments.length>1?arguments[1]:void 0,e.length)),o=c(t);return d?d.call(e,o,r):e.slice(r,r+o.length)===o}})}}]);
//# sourceMappingURL=chunk-8b77bb46.5fba90d6.js.map