.style_container_1v-vM{position:absolute;top:0;left:0;width:100%;height:100%;z-index:1000}.style_loader_3M6fk{display:block;position:relative;left:50%;top:calc(50% - 50px);width:150px;height:150px;margin:-75px 0 0 -75px;border-radius:50%;border:10px solid transparent;border-top-color:#ac262c;-webkit-animation:style_spin_M0-oL 2s linear infinite;animation:style_spin_M0-oL 2s linear infinite}.style_loader_3M6fk:before{content:"";position:absolute;top:5px;left:5px;right:5px;bottom:5px;border-radius:50%;border:10px solid transparent;border-top-color:#258e44;-webkit-animation:style_spin_M0-oL 3s linear infinite;animation:style_spin_M0-oL 3s linear infinite}.style_loader_3M6fk:after{content:"";position:absolute;top:20px;left:20px;right:20px;bottom:20px;border-radius:50%;border:10px solid transparent;border-top-color:#1c4678;-webkit-animation:style_spin_M0-oL 1.5s linear infinite;animation:style_spin_M0-oL 1.5s linear infinite}.style_message_1omzw{position:absolute;text-align:center;width:100%;top:calc(50% + 50px);font-size:200%;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}@-webkit-keyframes style_spin_M0-oL{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}@keyframes style_spin_M0-oL{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}